#include "Camera.h"
#include "Angel.h"

//---------------------------------------------------------------------------- constructor

Camera::Camera() :  fovy(45.0), aspect(1), zNear(0.5), zFar(100.0), eyeStart(vec4( 0.0 ,0.0, 10.0 ,2.0)),
                     VPN(vec4(0,0,1,0)), VUP(vec4(0,1,0,1))
{

    eye = eyeStart;  // We keep eyeStart around so the viewer can reset the camera

    // for mouse controls - tumble, dolly, track
    xStart = 0;  // used for to keep track of how much mouse is dragged
    yStart = 0;
    t = 0;  // Toggles the tumble point between the origin and fixed distance (d) from eye. Starts out at origin.
    d = 15; // The fixed distance.

    alpha = 1.0; // used to control the amount of a turn during the flythrough (see keyspecial)

    calcUVN();   //  calculates the viewRotation matrix based on VPN (view plane normal) and VUP (view Up direction)
}

//---------------------------------------------------------------------------- destructor
Camera::~Camera()
{}


void Camera::printCameraControls() {
    cout << "r ................. reset parameters" << "\n";
    cout << "Up arrow .......... turn up" << "\n";
    cout << "Down arrow ........ turn down" << "\n";
    cout << "Right arrow ....... turn right" << "\n";
    cout << "Insert key ........ roll left" << "\n";
    cout << "Page Up ........... roll right" << "\n";
    cout << "Home key .......... move camera forward" << "\n";
    cout << "End key ........... move camera backward" << "\n";
    cout << "Right Mouse drag .. pan up/down/left/right" << "\n";
    cout << "Middle Mouse drag . zoom in/out" << "\n";
    cout << "Left Mouse drag ... tumble about origin or white cube" << "\n";
    cout << "t ................. toggles the tumble point between origin and fixed distance"  << "\n";
    cout << "                    in front of camera. Starts at origin." << "\n";
}
//---------------------------------------------------------------------------- calcUVN
// calculate u, v, n from VPN and VUP
void Camera::calcUVN()
{
    //  NEED TO IMPLEMENT !!!
    // Be careful when doing matrix and vector operations with homogenous coordinates
    // For example, to cross two *vectors* (not points) stored as vec4's and to save result in a vec4 requires following:
    // a = vec4(cross(b,c),0);


}

//---------------------------------------------------------------------------- calcPerspective
// Computes the perpective matrix transform based on camera characteristics.
mat4 Camera::calcPerspective() {
   return Perspective( fovy, aspect, zNear, zFar );
}

//---------------------------------------------------------------------------- mouseControl
// Sets the state for determining whether the use wants to  Tumble, Dolly, or Track
void Camera::mouseControl(GLint button, GLint state, GLint x, GLint y ) {
 if (state == GLUT_DOWN)
    {
        switch (button)
        {
        case GLUT_LEFT_BUTTON:
            // cout << "     mouse: GLUT_LEFT_BUTTON - TUMBLE\n";
            action = TUMBLE;
            xStart = x;
            yStart = y;
            break;
        case GLUT_MIDDLE_BUTTON:
            //  cout << "     mouse: GLUT_MIDDLE_BUTTON - DOLLY\n";
            action = DOLLY;
            xStart = x;
            yStart = y;
            break;
        case GLUT_RIGHT_BUTTON:
            //  cout << "     mouse: GLUT_RIGHT_BUTTON - TRACK\n";
            action = TRACK;
            xStart = x;
            yStart = y;
            break;
        }
    }
}

//---------------------------------------------------------------------------- tumble
// tumble about to location tumblePoint in World Coordinate System.
void
Camera::tumble(mat4 rx, mat4 ry, vec4 tumblePoint)
{
    // We want to rotate about the world coordinate system along a direction parallel to the
    // camera's x axis. We first determine the coordinates of the WCS origin expressed in the eye coordinates.
    // We then translate this point to the camera (origin in camera coordinates) and do a rotation about x.
    // We then translate back. The result is then composed with the view matrix to give a new view matrix.
    //  NEED TO IMPLEMENT !!!   When done, should have new value for eye, u, v, n

    // DO THIS CONTROL LAST - IT IS THE MOST DIFFICULT PART


}

//---------------------------------------------------------------------------- motion
// Implements the track, dolly, and tumble controls (also see tumble method above)
void
Camera::motion( GLint x, GLint y )
{
    float dx, dy;
    mat4 ry, rx;
    vec4 tumblePoint;

    switch (action)
    {
    case TUMBLE:
        dx = 0.05*(x - xStart);
        dy = 0.05*(y - yStart);

        ry = RotateY(10*dx);
        rx = RotateX(10*dy);

        // tumble about a point tumblePoint in WCS. Two options currently.
        if (t==0)   // tumble about origin in  WCS
        {
            tumblePoint =  vec4(0,0,0,1);
        }
        else        // tumble about fixed distance in front of camera in  WCS
        {
            tumblePoint =   eye - d*viewRotation[2];
            tumblePoint[3] = 1;
        }
        tumble(rx, ry, tumblePoint);   //  <----  NEED TO IMPLEMENT this method!!!

        xStart = x;
        yStart = y;
        break;
    case TRACK:  // PAN
        dx = -0.05*(x - xStart);
        dy = 0.05*(y - yStart);
        //  NEED TO IMPLEMENT !!!

        break;
    case DOLLY:
        dy = 0.05*(y - yStart);
        dx = 0.05*(x - xStart);
        //  NEED TO IMPLEMENT !!!

        break;
    default:
        printf("unknown action %d\n", action);
    }

    glutPostRedisplay();
}

//---------------------------------------------------------------------------- reset
// reset all of the camera parameters
void
Camera::reset() {
    t = 0;  // tumble point at origin
    eye = eyeStart; // camera location
    calcUVN();
    xStart = 0.0;
    yStart = 0.0;
}

//---------------------------------------------------------------------------- keyboard
// toggle the tumble point: tumbles either around the world space origin (0) or
//         tumbles a fixed distance in from of the camera (1).
void
Camera::toggle() {
   t = (t+1)%2; // toggle tumble point
}

//---------------------------------------------------------------------------- keyboard

//---------------------------------------------------------------------------- keySpecial
// Controls fly-through
void
Camera::keySpecial( int key, int x, int y )
{
    switch( key )
    {
    case GLUT_KEY_UP:   // rotate around y
        //  cout << " you pressed  up \n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_DOWN:     // rotate around x  (this is always applied second)
        //  cout << " you pressed  down \n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_LEFT:     // rotate around z  (this is always applied second)
        //   cout << " you pressed  left \n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_RIGHT:
        //   cout << " you pressed  right\n";
        // cout << "u = " << u << "\n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_HOME: // forward
        //   cout << " you pressed  forward\n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_END:
        //  cout << " you pressed  backward\n";
        //  NEED TO IMPLEMENT !!!

        break;
    case GLUT_KEY_INSERT:
        //   cout << " you pressed bank left \n";
        //  NEED TO IMPLEMENT !!!

        break;
    case  GLUT_KEY_PAGE_UP :
        //  cout << " you pressed bank right\n";
        //  NEED TO IMPLEMENT !!!

        break;
    }

    glutPostRedisplay();
}


