#ifndef CAMERA_H
#define CAMERA_H

#include "Angel.h"

class Camera
{
public:
    Camera();
    virtual ~Camera();

    // Camera projection transformation parameters
    GLfloat  fovy;         // Field-of-view in Y direction angle (in degrees)
    GLfloat  aspect;       // Viewport aspect ratio
    GLfloat  zNear;        // camera's far plane
    GLfloat zFar;          // camera's near plane

// Camera location and orientation parameters
    vec4 eyeStart; // initial camera location
    vec4 eye; // camera location
    vec4 VPN;  // used to initialize uvn
    vec4 VUP;  // used to initialize uvn
    mat4 viewRotation;  // rotational part of matrix that transforms between World and Camera coordinates

    void calcUVN();
    mat4 calcPerspective();
    void mouseControl(GLint button, GLint state, GLint x, GLint y );
    void keySpecial( int key, int x, int y );
    void motion( GLint x, GLint y );
    void tumble(mat4 rx, mat4 ry, vec4 tumblePoint);
    void printCameraControls();
    void reset();
    void toggle();
protected:
public:
    // These are for the camera mouse controls
    enum  actions { TUMBLE, TRACK, DOLLY };
    GLint   action;
    int   xStart;
    int   yStart;
    int t;        // toggles the tumble point between the origin and fixed distance (d) from eye. Starts out at origin
    float d;      // fixed distance
    float alpha;  // used to control the amount of a turn during the flythrough
};

#endif // CAMERA_H
